//-----------------------------------------------------------------------------
//
//	Musepack Decoder
//
//	Author : Igor Janos
//
//-----------------------------------------------------------------------------
#include "stdafx.h"


void MakeNiceSpeed(__int64 bps, CString &v)
{
	int r=0;
	__int64	c=bps;
	LPCTSTR		rady[] = {
		_T("bps"),
		_T("kbps"),
		_T("mbps"),
		_T("gbps"),
		_T("tbps")
	};

	// spocitame rad
	while (c > 1000 && r<4) {
		r++;
		c = c / 1000;
	}

	c=bps;
	for (int i=1; i<r; i++) { c = c/1000; }
	double d=c / 1000.0;

	v.Format(_T("%5.3f %s"), (float)d, rady[r]);
}

//-----------------------------------------------------------------------------
//
//	CMPCDecoder class
//
//-----------------------------------------------------------------------------

CUnknown *CMPCDecoder::CreateInstance(LPUNKNOWN pUnk, HRESULT *phr)
{
	return new CMPCDecoder(pUnk, phr);
}

CMPCDecoder::CMPCDecoder(LPUNKNOWN pUnk, HRESULT *phr) :
	CTransformFilter(_T("MPC Decoder"), pUnk, CLSID_MusepackDecoder)
{
	m_pInput = new CTransformInputPin(NAME("Input"), this, phr, L"Input");
	m_pOutput = new CTransformOutputPin(NAME("Output"), this, phr, L"Output");
}

CMPCDecoder::~CMPCDecoder()
{
}

STDMETHODIMP CMPCDecoder::NonDelegatingQueryInterface(REFIID riid,void **ppv)
{
    CheckPointer(ppv,E_POINTER);

	if (riid == IID_ISpecifyPropertyPages) {
		return GetInterface((ISpecifyPropertyPages*)this, ppv);
	} else
		return CBaseFilter::NonDelegatingQueryInterface(riid,ppv);
}

STDMETHODIMP CMPCDecoder::GetPages(CAUUID *pPages)
{
    CheckPointer(pPages,E_POINTER);

    pPages->cElems = 1;
    pPages->pElems = (GUID *) CoTaskMemAlloc(sizeof(GUID));
    if (pPages->pElems == NULL) {
        return E_OUTOFMEMORY;
    }

	*(pPages->pElems) = CLSID_MusepackDecoderPage;
    return NOERROR;

} // GetPages

HRESULT CMPCDecoder::CheckInputType(const CMediaType *mtIn)
{
	// we only accept these
	if (mtIn->majortype != MEDIATYPE_Audio) return E_FAIL;
	if (mtIn->subtype != MEDIASUBTYPE_MusepackPacket) return E_FAIL;
	if (mtIn->formattype != FORMAT_WaveFormatEx) return E_FAIL;

	return NOERROR;
}

HRESULT CMPCDecoder::CheckTransform(const CMediaType *mtIn, const CMediaType *mtOut)
{
	HRESULT hr = CheckInputType(mtIn);
	if (FAILED(hr)) return hr;

	// output is fixed
	if (mtOut->majortype != MEDIATYPE_Audio) return E_FAIL;
	if (mtOut->subtype != MEDIASUBTYPE_PCM) return E_FAIL;

	return NOERROR;
}

HRESULT CMPCDecoder::SetMediaType(PIN_DIRECTION direction, const CMediaType *pmt)
{
	if (direction == PINDIR_INPUT) {
		if (pmt->formattype != FORMAT_WaveFormatEx) return E_FAIL;

		WAVEFORMATEX *wfx = (WAVEFORMATEX*)pmt->pbFormat;

		// we assemble output format
		wfOut.cbSize = 0;
		wfOut.nChannels = wfx->nChannels;
		wfOut.nSamplesPerSec = wfx->nSamplesPerSec;
		wfOut.wFormatTag = WAVE_FORMAT_PCM;
		wfOut.wBitsPerSample = 16;				// currently only 16-bit
		wfOut.nBlockAlign = (wfOut.wBitsPerSample * wfOut.nChannels) >> 3;
		wfOut.nAvgBytesPerSec = wfOut.nBlockAlign * wfOut.nSamplesPerSec;

	}
	return NOERROR;
}

HRESULT CMPCDecoder::BreakConnect(PIN_DIRECTION dir)
{
	if (dir == PINDIR_INPUT) {
		memset(&wfOut, 0, sizeof(WAVEFORMATEX));

		// disconnect output
		if (m_pOutput->IsConnected()) {
			m_pOutput->GetConnected()->Disconnect();
			m_pOutput->Disconnect();
		}
	}
	return __super::BreakConnect(dir);
}

HRESULT CMPCDecoder::DecideBufferSize(IMemAllocator *pAlloc, ALLOCATOR_PROPERTIES *pProp)
{
	// since the frames are of fixed size we can be sure
	// that the size won't be larger than....
	pProp->cbBuffer = (wfOut.nBlockAlign * 1152) * 1.5;		// + some extra space
	pProp->cBuffers = 3;

	ALLOCATOR_PROPERTIES	act;
	HRESULT hr = pAlloc->SetProperties(pProp, &act);
	if (FAILED(hr)) return hr;

	if (act.cbBuffer < pProp->cbBuffer) return E_FAIL;
	return NOERROR;
}

HRESULT CMPCDecoder::GetMediaType(int iPosition, CMediaType *pmt)
{
	if (iPosition < 0) return E_INVALIDARG;
	if (iPosition > 0) return VFW_S_NO_MORE_ITEMS;

	// don't accept connections until input pin is connected
	if (m_pInput->IsConnected() == FALSE) return E_FAIL;

	pmt->majortype  = MEDIATYPE_Audio;
	pmt->subtype    = MEDIASUBTYPE_PCM;
	pmt->formattype	= FORMAT_WaveFormatEx;

	// we already know...
	WAVEFORMATEX	*pwf = (WAVEFORMATEX*)pmt->AllocFormatBuffer(sizeof(WAVEFORMATEX));
	memcpy(pwf, &wfOut, sizeof(WAVEFORMATEX));

	return NOERROR;
}

HRESULT CMPCDecoder::GetDeliveryBuffer(IMediaSample **sample)
{
    IMediaSample *pOutSample;
    HRESULT hr = m_pOutput->GetDeliveryBuffer(&pOutSample, NULL, NULL, 0);
    *sample = pOutSample;
    if (FAILED(hr)) {
        return hr;
    }

	// check for type change
	AM_MEDIA_TYPE *mt;
	if (pOutSample->GetMediaType(&mt) == NOERROR) {
		CMediaType _mt(*mt);
		SetMediaType(PINDIR_OUTPUT, &_mt);
		DeleteMediaType(mt);
	}

	return NOERROR;
}

HRESULT CMPCDecoder::StartStreaming()
{
	HRESULT hr = __super::StartStreaming();
	if (FAILED(hr)) return hr;
	return hr;
}

HRESULT CMPCDecoder::StopStreaming()
{
	HRESULT hr = __super::StopStreaming();
	if (FAILED(hr)) return hr;
	return hr;
}

HRESULT CMPCDecoder::Receive(IMediaSample *pSample)
{
	if (!m_pOutput->IsConnected()) return NOERROR;

	// todo:
	return NOERROR;
}



